/***************************************************************************
 *   Copyright (C) 2005 by Dmitry Nezhevenko                               *
 *   dionua@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** \example p2ktest.c
 * This is an example of how to use the p2kmoto library.
 */

#include "../src/p2kmoto.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void onFile(struct p2k_fileInfo nfo)
{
	printf("%4ld %8ld %3d %3d %s\n",nfo.id, nfo.size, nfo.owner, nfo.attr, nfo.name );
}

char s[1024];
size_t ps;

int getArg(char *ss)
{
	int pps=0;
	while (ps<strlen(s))
	{
		if (s[ps]!=' ') ss[pps++]=s[ps]; else break;
		ps++;
	}
	ps++;
	ss[pps]=0;
	return (pps);	
}

void usage(char commandsOnly)
{
	if (!commandsOnly)
	{
		printf("usage: p2ktest [optioms] [command]\n");
		printf("  -l		show all usb devices\n");
		printf("  -d device	set ACM device\n");
		printf("  -a vid:pid	set USB device (AT)\n");
		printf("  -p vid:pid	set USB device (P2K)\n");
		printf("  -h		show this help\n");
	}
	printf("commands:\n");
	printf("  version	show phone FSAC version\n");
	printf("  model		show phone model\n");
	printf("  volumes	show available volumes\n");
	printf("  ls [mask]	show file list\n");
	printf("  reboot	reboot phone\n");
}

void doVersion()
{
	char s[256];
	if (!p2k_FSAC_getVersion(s))
		printf("FSAC version: %s\n",s);
	else
		printf("Can not get FSAC version\n");
}

void doModel()
{
	unsigned char s[256];
        if (!p2k_getPhoneModel(s))
                printf("Phone Model: %s\n", s);
        else
                printf("Can not get phone model\n");
}

void doVolumes()
{
	char s[256];
	if (!p2k_FSAC_getVolumes(s))
		printf("Volumes: %s\n", s);
	else
		printf("Can not get volume names\n");
}

void doLs(char* request)
{
	p2k_FSAC_searchRequest(request);
	p2k_FSAC_fileList(onFile);
}

void doReboot()
{
	printf("Rebooting phone\n");
	p2k_reboot();
}

void runCmd(int cnt, char* cmd[])
{
	char s[256];
	if (!cnt)
		return;
	if (!strcmp(cmd[0],"help"))
		usage(1);
	else if (!strcmp(cmd[0],"version"))
		doVersion();
	else if (!strcmp(cmd[0],"model"))
		doModel();
	else if (!strcmp(cmd[0],"volumes"))
		doVolumes();
	else if (!strcmp(cmd[0],"ls"))
	{
		if (cnt==2)
			strncpy(s,cmd[1],sizeof(s));
		else
			strncpy(s,"*",sizeof(s));
		doLs(s);
	}
	else if (!strcmp(cmd[0],"reboot"))
		doReboot();
	else
		printf("Unknown command\n");
}

void dialog(void)
{
	char *cmd[100];
	int i,lastI,l,cmdCnt;
	printf("Enter \"help\" to see help\n");
	while (1)
	{
		printf("> ");
		if (!fgets(s, sizeof(s), stdin))
		{
			printf("\n");
			return;
		}
		l=strlen(s);
		cmdCnt=0;
		lastI=0;
		cmd[cmdCnt++]=s;
		for (i=0; i<l; i++)
		{
			if (s[i]=='\r' || s[i]=='\n' || s[i]==' ')
			{
				s[i]=0;
				if (i+1<l)
					cmd[cmdCnt++]=&s[i+1];
			}
		}
		runCmd(cmdCnt, cmd);
	}
}


int main(int argc, char *argv[])
{
	p2k_init();
	int i;
	char listDevices=0;
	char needHelp=0;
	char argError=0;
	char noDetect=0;
	char argBuf[256];
	char *argPtr;
	char *cmd[100];
	int cmdCnt=0;
	int argInt1, argInt2;

	for (i=1; i<argc; i++)
	{
		if (argError)
			break;
		if (!strcmp(argv[i],"-l"))
			listDevices=1;
		if (!strcmp(argv[i],"-d"))
		{
			i++;
			if (i<argc)
				p2k_setACMdevice(argv[i]);
			else
				argError=1;
			continue;
		} else
		if (!strcmp(argv[i],"-a"))
		{
			noDetect=1;
			i++;
			if (i<argc)
			{
				strncpy(argBuf,argv[i],sizeof(argBuf));
				argPtr=strchr(argBuf,':');
				if (!argPtr)
				{
					argError=1;
				}
				else
				{
					*argPtr=' ';
					sscanf(argBuf,"%x%x", &argInt1, &argInt2);
					p2k_setATconfig(argInt1, argInt2);
					p2k_setP2Kconfig(argInt1, argInt2-1);
				}
			
			}
			else
				argError=1;
			continue;
		} else
		if (!strcmp(argv[i],"-p"))
		{
			noDetect=1;
			i++;
			if (i<argc)
			{
				strncpy(argBuf,argv[i],sizeof(argBuf));
				argPtr=strchr(argBuf,':');
				if (!argPtr)
				{
					argError=1;
				}
				else
				{
					*argPtr=' ';
					sscanf(argBuf,"%x%x", &argInt1, &argInt2);
					p2k_setP2Kconfig(argInt1, argInt2);
				}
			
			}
			else
				argError=1;
			continue;
		} else
		if (!strcmp(argv[i],"-h") || (!strcmp(argv[i],"--help")))
		{
			needHelp=1;
		}
		else
		{
			cmd[cmdCnt++]=argv[i];
		}
	}
	printf("P2k Test program\n\n");

	if (needHelp)
	{
		usage(0);
		return 0;
	}

	if (listDevices)
	{
		printf("Device list:\n");
		struct p2k_devInfo * nfo;

		nfo=p2k_getDevList();
		i=0;
		while (nfo[i].vendor>=0)
		{
			printf("  %04x:%04x: [%s] [%s]\n", nfo[i].vendor, nfo[i].product,
			       nfo[i].manufacturerStr, nfo[i].productStr);
			i++;
		}
		free(nfo);
		printf("\n");
	}
	
	if (!noDetect)
	{
		printf("Detecting phone... ");
		if (p2k_detectPhone()>=0)
			printf("phone detected\n");
		else
			printf("no phone detected\n");
	}

	p2k_getACMdevice(argBuf);
	printf("Using:\n");
	printf("  ACM device : %s\n",argBuf);
	printf("  AT device  : %04x:%04x\n", p2k_getATvendor(), p2k_getATproduct());
	printf("  P2K device : %04x:%04x\n", p2k_getP2Kvendor(), p2k_getP2Kproduct());

	int state=p2k_findPhone();

	if (state==P2K_PHONE_AT)
	{
		//We must switch phone to p2k mode by sending AT+MODE=8 command.
		printf("Switching to P2K...\n");

		if (p2k_setP2Kmode(10)<=0)
		{
			printf("Unable to switch mode\n\n");
			return 0;
		}
		printf("  done.\n");
		state=p2k_findPhone();
	}

	if (state==P2K_PHONE_NONE)
	{
		printf("No phone found.\n");
		return 0;
	}
	
	printf("P2k Phone found. Opening...\n");
	p2k_openPhone(3000);
	printf("  done.\n");
	
	if (!cmdCnt)
		dialog();
	else
		runCmd(cmdCnt, cmd);
	
	return 0;

	long sz;
	char s[200];
	if (!p2k_getPhoneModel((unsigned char *)s))
		printf("Phone Model: %s\n", s);
	else
		printf("Can not get phone model\n");
	
	if (!p2k_FSAC_getVersion(s))
		printf("FSAC version: %s\n",s);
	else
		printf("Can not get FSAC version\n");
	
	if (!p2k_FSAC_getVolumes(s))
		printf("Drives: %s\n", s);
	else
		printf("Can not get drive name\n");

	dialog();
	p2k_closePhone();
	return 0;
}

